/**
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package com.droidlar;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.Timer;

/**
 * Classe responsável pelo gerenciamento das threads que executaram os perfis.
 */
public class AgendadorDePerfis{

    private Timer timerInicio;
    private Timer timerFim;

    private int horaInicial;
    private int minutoInicial;
    private int horaFinal;
    private int minutoFinal;
    private int[] diasDaSemana;

    private ArrayList<Timer> timers;
    private InfoDisp infoDisp;
    private Servidor root;

    private static final long umaSemana = 1000*60*60*24*7;

    public AgendadorDePerfis(Servidor r, InfoDisp inf) {

        root = r;
        infoDisp = inf;
        timers = new ArrayList<Timer>();

        horaInicial = infoDisp.ini_hora;
        minutoInicial = infoDisp.ini_min;
        horaFinal = infoDisp.fim_hora;
        minutoFinal = infoDisp.fim_min;
        diasDaSemana = infoDisp.dias;

        this.agendar();
    }

    /**
     * Agenda threads que executaram os perfis.
     */
    private void agendar() {
        if (diasDaSemana.length > 0) {

            // verifica se é uma data específica ex: 1 de janeiro de 2020,
            // se for, a thread será executada um única vez
            if (diasDaSemana[0] == -1) {
                if (diasDaSemana.length == 4) {

                    timerInicio = new Timer();
                    timerFim = new Timer();

                    timers.add(timerInicio);
                    timers.add(timerFim);

                    int diaDoMes = diasDaSemana[1];

                    Date inicio = getDate(horaInicial, minutoInicial, -1, diaDoMes, diasDaSemana[2], diasDaSemana[3]);
                    
                    if ((horaInicial > horaFinal) || ((horaInicial == horaFinal) && (minutoInicial >= minutoFinal))) {
                        diaDoMes++;
                    }

                    Date fim = getDate(horaFinal, minutoFinal, -1, diaDoMes, diasDaSemana[2], diasDaSemana[3]);

                    Calendar calendar = Calendar.getInstance();
                    Date agora = calendar.getTime();

                    if (fim.after(agora)) {
                        System.out.println("" + inicio + "\n" + fim);

                        // Dispara tarefa inicial
                        Tarefa tarefaInicial = new Tarefa(root, infoDisp.valorInicial, infoDisp.ID, infoDisp.endereco, timerInicio);
                        timerInicio.schedule(tarefaInicial, inicio);

                        // Dispara tarefa final
                        Tarefa tarefaFinal = new Tarefa(root, infoDisp.valorFinal, infoDisp.ID, infoDisp.endereco, timerFim);
                        timerFim.schedule(tarefaFinal, fim);
                    }
                }

            } else {
                // caso a data sejam dias da semana as threads serão executadas repetidamente

                Calendar calendar = Calendar.getInstance();
                int diaHojeMes = calendar.get(Calendar.DAY_OF_MONTH);
                Date agora = calendar.getTime();

                for (int i = 0; i < diasDaSemana.length; i++) {

                    // timers que irão irão agendar as threads
                    timerInicio = new Timer();
                    timerFim = new Timer();

                    // adiciona os timers na lista que será usada para finalizar as threads
                    timers.add(timerInicio);
                    timers.add(timerFim);

                    int diaDaSemanaFim = diasDaSemana[i];
                    int diaHojeMesFim = diaHojeMes;

                    // verifica se horário de fim seja igual ou anterior ao horário de início,
                    // caso seja, incrementaem em 1 o dia da semana da tarefa final
                    if ((horaInicial > horaFinal) || ((horaInicial == horaFinal) && (minutoInicial >= minutoFinal))) {
                        diaDaSemanaFim++;
                        diaHojeMesFim++;
                    }

                    Date inicio = getDate(horaInicial, minutoInicial, diasDaSemana[i], -1, -1, -1);
                    Date fim = getDate(horaFinal, minutoFinal, diaDaSemanaFim, -1, -1, -1);

                    // caso a data final já tenha passado altera ela para a próxima semana
                    if (fim.before(agora)) {
                        int diaHoje = calendar.get(Calendar.DAY_OF_WEEK);
                        if (diasDaSemana[i] == diaHoje) {
                            inicio = getDate(horaInicial, minutoInicial, diasDaSemana[i], diaHojeMes + 7, -1, -1);
                            fim = getDate(horaFinal, minutoFinal, diaDaSemanaFim, diaHojeMesFim + 7, -1, -1);
                        } else if (diaDaSemanaFim == diaHoje) {
                            fim = getDate(horaFinal, minutoFinal, diaDaSemanaFim, diaHojeMes + 7, -1, -1);
                        }
                    }

                    System.out.println("" + inicio);
                    System.out.println("" + fim);

                    // Dispara tarefa inicial
                    Tarefa tarefaInicial = new Tarefa(root, infoDisp.valorInicial, infoDisp.ID, infoDisp.endereco, null);
                    timerInicio.schedule(tarefaInicial, inicio, umaSemana);

                    // Dispara tarefa final
                    Tarefa tarefaFinal = new Tarefa(root, infoDisp.valorFinal, infoDisp.ID, infoDisp.endereco, null);
                    timerFim.schedule(tarefaFinal, fim, umaSemana);
                }
            }
        }
    }

    /**
     * Monta um objeto Date a partir de um conjunto de valores passados como parametros.
     * @param h hora
     * @param m minuto
     * @param ds dia da semana
     * @param dm dia do mês
     * @param ma mês
     * @param an ano
     * @return data resultante dos valores dos parametros
     */
    public Date getDate(int h, int m, int ds, int dm, int ma, int an){

        Calendar calendar = Calendar.getInstance();
        
        calendar.setFirstDayOfWeek(calendar.get(Calendar.DAY_OF_WEEK));
        calendar.set(Calendar.HOUR_OF_DAY, h);
        calendar.set(Calendar.MINUTE, m);
        calendar.set(Calendar.SECOND, 0);

        if(ds != -1){
            calendar.set(Calendar.DAY_OF_WEEK, ds);
        }

        if(dm != -1){
            calendar.set(Calendar.DAY_OF_MONTH, dm);
        }

        if(ma != -1){
            calendar.set(Calendar.MONTH, ma);
            calendar.set(Calendar.YEAR, an);
        }

        return calendar.getTime();
    }

    /**
     * Finaliza todas as threads agendadas que executariam as tarefas dos perfis.
     */
    public void mataPerfil() {
        for(Timer t : timers){
            t.cancel();
        }
        System.out.println("TAREFA MORTA!");
    }
}
