/**
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2009 Michel Vinicius de Melo Euzebio e Emerson Ribeiro de Mello
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package com.droidlar;

import com.rapplogic.xbee.api.XBeeAddress64;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.PrintWriter;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

/**
 * Classe responsável pela recepção e gerenciamento das mesagens HTTP
 * enviadas pela aplicação cliente do DriodLar.
 */
public class Servidor extends HttpServlet {

    private GererenciadorDosModulos gerModulos;
    private Infos perfis;
    private String[] uNome = new String[2];
    private String[] uSenha = new String[2];

    public static final String MSG_SUCESSO = "0";
    public static final String MSG_ERRO = "1";

    public static final int MSG_AUTENTICACAO = 2;
    public static final int MSG_CONTROLE = 3;
    public static final int REQUEST_DISP = 4;
    public static final int MSG_VARREDURA = 5;
    public static final int MSG_PERFIL = 6;
    public static final int REQUEST_POR_TIPO = 7;
    public static final int MSG_SET_NOME = 11;
    public static final int REQUEST_PERFIL_UUID = 12;
    public static final int REQUEST_PERFIL = 13;
    public static final int SET_VARREDURA = 14;

    public Servidor(){

        gerModulos = new GererenciadorDosModulos(this);

        // Restaura o objeto perfis e agenda os perfis
        perfis = (Infos) recuperarObjeto("perfis.obj");
        
        if(perfis == null){
            perfis = new Infos();
        }else{
            perfis.agendaTarefas(this);
        }

        // Configura atributos com nomes de usuário e senha

        // Super Usuário
        uNome[0] = "admin";
        uSenha[0] = "admin";

        // Usuário comum
        uNome[1] = "android";
        uSenha[1] = "android";
    }

    /** 
     * Processes requests for both HTTP <code>GET</code> and <code>POST</code> methods.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    protected void processRequest(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {

        HttpSession session = request.getSession(true);
        System.out.println(session.getId());

        response.setContentType("text/plain;charset=UTF-8");
        PrintWriter out = response.getWriter();

        try {

            String tipo = request.getParameter("tipo");

            if (tipo != null) {

               if (Integer.parseInt(tipo) == MSG_AUTENTICACAO) {

                    String usuario = request.getParameter("usuario");
                    String senha = request.getParameter("senha");

                    if ((usuario != null) && (senha != null)) {
                        if (usuario.equals(uNome[0]) && senha.equals(uSenha[0])) {

                            out.println(MSG_SUCESSO);
                            session.setAttribute("autentico", true);

                        }else if (usuario.equals(uNome[1]) && senha.equals(uSenha[1])) {

                            out.println(MSG_SUCESSO);
                            session.setAttribute("autentico", false);
                        }else{
                            out.println(MSG_ERRO);
                        }
                    }else{
                        out.println(MSG_ERRO);
                    }

                } else if (session.getAttribute("autentico") != null) {

                    String modulo = request.getParameter("modulo");
                    String mensagem = request.getParameter("mensagem");
                    String opcao = request.getParameter("opcao");

                    if ((modulo != null) && (opcao != null) && (mensagem != null) && (tipo != null)) {

                        switch(Integer.parseInt(tipo)){

                            // mensagens de controle sao para o ligamento/desligamento dos dispositivos
                            case MSG_CONTROLE:

                                if (gerModulos.enviarDados(tipo, mensagem, opcao, modulo) == true) {
                                    out.println(MSG_SUCESSO);
                                } else {
                                    out.println(MSG_ERRO);
                                }
                                break;

                            // mensagem de requisicao da lista de dispositivos descobertos
                            case REQUEST_DISP:

                                out.println(gerModulos.getDisps());
                                break;

                            case REQUEST_POR_TIPO:

                                out.println(gerModulos.getPorTipo(mensagem));
                                break;

                            case MSG_SET_NOME:

                                // Verifica se usuário é super usuário
                                if(((Boolean)session.getAttribute("autentico")) == true){                                 
                                    gerModulos.setNomeDisp(mensagem);
                                    out.println(MSG_SUCESSO);
                                }else{
                                    out.println(MSG_ERRO);
                                }
                                break;

                            case REQUEST_PERFIL_UUID:

                                out.println(perfis.getUUID());
                                break;

                            case REQUEST_PERFIL:

                                out.println(perfis.getPerfis());
                                break;

                            case MSG_PERFIL:

                                out.println(MSG_SUCESSO);
                                perfis.gerarInfos(this, mensagem);
                                break;

                            case SET_VARREDURA:

                                // Verifica se usuário é super usuário
                                if(((Boolean)session.getAttribute("autentico")) == true){                                    
                                    gerModulos.gerVarredura(mensagem);
                                    out.println(MSG_SUCESSO);
                                }else{
                                    out.println(MSG_ERRO);
                                }
                                break;

                            case MSG_VARREDURA:

                                // verifica se o módulo xbee está conectado ao SAR
                                if (gerModulos.getXbee().isConnected()) {

                                    // envia mensagem de varredura
                                    gerModulos.getThreadVar().verredura();

                                    // espera por um tempo para receber a resposta assincrona
                                    try {
                                        Thread.sleep(1000);
                                    } catch (InterruptedException ex) {
                                        Logger.getLogger(GererenciadorDosModulos.class.getName()).log(Level.SEVERE, null, ex);
                                    }
                                }

                                out.println(MSG_SUCESSO);                            
                        }
                                               
                    }

                } else {
                    out.println(MSG_ERRO);
                }

            }

        } finally {
            out.close();
        }

    }

    /**
     * Restaura objetos gravados em arquivos.
     * @param fileName nome do arquivo contendo o objeto
     * @return objeto restaurado
     */
    public Object recuperarObjeto(String fileName){

        File arquivo = new File(fileName);
        Object objeto = null;

        if (arquivo.exists()) {
            try {
                FileInputStream fin = new FileInputStream(arquivo);
                ObjectInputStream oin = new ObjectInputStream(fin);

                // Lendo os objetos de um arquivo e fazendo a conversao de tipos
                objeto = oin.readObject();

                oin.close();
                fin.close();

            } catch (Exception ex) {
                Logger.getLogger(Servidor.class.getName()).log(Level.SEVERE, null, ex);
            }
        }

        return objeto;
    }

    /**
     * Grava objetos em arquivos.
     * @param objeto objeto que será gravado
     * @param fileName nome do arquivo onde o objeto será gravado
     */
    public void gravarObjeto(Object objeto, String fileName){
        File arquivo = new File(fileName);

        try {

            FileOutputStream fout = new FileOutputStream(arquivo);
            ObjectOutputStream oout = new ObjectOutputStream(fout);

            // gravando o objeto no arquivo
            oout.writeObject(objeto);

            oout.flush();
            oout.close();
            fout.close();

        } catch (Exception ex) {
            Logger.getLogger(Servidor.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    /**
     * @return objeto que gerencia a comunicação com os módulos xBee
     */
    public GererenciadorDosModulos getGerModulos() {
        return gerModulos;
    }

    /**
     * Converte um endereço de um módulo xBee no formato String para o formato XBeeAddress64.
     * @param end endereco no formato String
     * @return endereco no formato XBeeAddress64
     */
    public XBeeAddress64 convertStringToX64(String end){
        // cria vetor com o endereco
        String[] bytes = end.split(",");
        int[] n = new int[bytes.length];
        for (int j = 0; j < bytes.length; j++) {
            n[j] = Integer.parseInt(bytes[j], 16);
        }

        return new XBeeAddress64(n);
    }

    /**
     * Converte um endereço de um módulo xBee no formato XBeeAddress64 para o formato String.
     * @param end endereco no formato XBeeAddress64
     * @return endereco no formato String
     */
    public String convertX64ToString(XBeeAddress64 end){

        String[] partes = (""+end).split(",");
        String endConv = "";
        for(String temp : partes){
            if(temp.substring(2,3).equals("0")){
                endConv += ","+temp.substring(3);
            }else{
                endConv += ","+temp.substring(2);
            }
        }
        
        return endConv.substring(1);
    }

    // <editor-fold defaultstate="collapsed" desc="HttpServlet methods. Click on the + sign on the left to edit the code.">
    /** 
     * Handles the HTTP <code>GET</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        processRequest(request, response);
    } 

    /** 
     * Handles the HTTP <code>POST</code> method.
     * @param request servlet request
     * @param response servlet response
     * @throws ServletException if a servlet-specific error occurs
     * @throws IOException if an I/O error occurs
     */
    @Override
    protected void doPost(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException {
        processRequest(request, response);
    }

    /** 
     * Returns a short description of the servlet.
     * @return a String containing servlet description
     */
    @Override
    public String getServletInfo() {
        return "Short description";
    }// </editor-fold>

}
